<?php
/* 	
 * 	Plugin Name: Attachment File Icons (AF Icons)
 *	Plugin URI: 
 *	Description: A plugin to display file type icons adjacent to files uploaded to pages/posts. Feature to upload icons for different file types provided. Please refer plugin overview page for details.
 *	Author: Praveen Rajan
 *	Version: 1.0 
 *	Author URI: 
 * 
*/
if (!class_exists("AttachmentFileIcons"))
{
	class AttachmentFileIcons {
		
		//stores plugin url
		var $plugin_url;
		var $plugin_path;
		var $afi_table;
		
		/**
		 * Constructor of plugin
		 * @return void
		 * @author Praveen Rajan
		 */
		function AttachmentFileIcons(){
			
			$this->plugin_url = WP_CONTENT_URL.'/plugins/'.plugin_basename(dirname(__FILE__));
			$this->plugin_path = WP_CONTENT_DIR . '/plugins/' .	dirname( plugin_basename(__FILE__))  ;
			$this->afi_table = '';
			 
			//adds admin menu options to manage
			add_action('admin_menu', array(&$this, 'admin_menu'));
			
			//adds option to pages/posts content
			add_filter('the_content', array(&$this, 'Attachment_File_Parse'));
			
			//adds scripts and css stylesheets
			add_action('wp_print_scripts', array(&$this, 'Attachment_Header_Code'));
			
			//adds contextual help
			add_action('admin_init',  array(&$this, 'add_gallery_contextual_help'));
		}
		
		/**
		 * Function to add main menu and submenus to admin panel
		 * @return adds menu
		 * @author Praveen Rajan
		 */
		function admin_menu() {
			
			add_menu_page('Attachment File Icons', 'AF Icons', 'manage_options', 'afi-overview' , array(&$this, 'afi_overview'), $this->plugin_url .'/afi_small.png');
			add_submenu_page( 'afi-overview', 'Attachment File Icons Overview', 'Manage', 'manage_options', 'afi-overview',array(&$this, 'afi_overview'));
			add_submenu_page( 'afi-overview', 'Add Attachment Icons', 'Add Icons', 'manage_options', 'afi-add',array(&$this, 'afi_add'));
		}
		
		/**
		 * Function to add contextual help for plugin pages.
		 * @author Praveen Rajan
		 */
		function add_gallery_contextual_help() {
			
			$help_content = '<p><b>Instructions to use Attachment File Icons<i>(AF Icons)</i>:</b></p>';
			$help_content .= '<p><ol><li>Upload file to posts/pages using media upload icon.</li>'.
							 '<li>Use button \'Insert into post\' to add uploaded file to post content.</li>'.	
							 '<li>Add the following syntax with the link inserted from the above step.</li>'.		
							 '<p style="padding-left:40px;"><b>[attachment link=&lt;link&gt; title=\'&lt;title of document&gt;\' /]</b></p>'.
							 '<li>View the page to see the effect of AF Icons*.</li></ol></p>'.
							 '<p style="padding-left:40px;">Sample Preview: <a href="#"><img src="' . $this->plugin_url . '/afi_small.png" /></a> <a href="#">AF Icon File</a></p>';
			add_contextual_help( 'toplevel_page_afi-overview', $help_content );
			add_contextual_help( 'af-icons_page_afi-add', $help_content );
		}
		
		/**
		 * Function to show overview page of plugin
		 * @author Praveen Rajan
		 */
		function afi_overview(){
			
			$title = __('AF Icons Overview - Available Icons');
			global $wpdb;
			$sub_name_type = 'afi_types';
	        $this->afi_table  = $wpdb->prefix . $sub_name_type;
	        
			//Section to delete a single icon
			if(isset($_POST['TB_iconsingle']) && !empty($_POST['TB_iconsingle']) && $_POST['TB_DeleteSingle'] == 'OK') {
				$id = $_POST['TB_iconsingle'];
				$icon_result = $wpdb->get_var("SELECT icon_name FROM " .  $this->afi_table . " WHERE id = '$id' ");
				$status = false;
				if (function_exists('is_multisite') && is_multisite()) {
					
					$old_blog = $wpdb->blogid;
					// Get all blog ids
					$blogids = $wpdb->get_col($wpdb->prepare("SELECT blog_id FROM $wpdb->blogs"));
					foreach ($blogids as $blog_id) {
						switch_to_blog($blog_id);
						AttachmentFileIcons::_afi_delete($id);
					}
					switch_to_blog($old_blog);
					$status = true;
				}else {
					$wpdb->query("DELETE FROM $this->afi_table WHERE id = '$id'");
					$status = true;	
				}
				
				if(	$status ){
					@unlink($this->plugin_path . '/mime/' . $icon_result);
				}
				echo '<div class="clear" style="min-height:15px;"></div><div class="wrap"><div id="message" class="updated fade below-h2"><p>Icon ' . $icon_result . ' deleted successfully.</p></div></div>';
			}
			$icon_types = $wpdb->get_results("SELECT * FROM  $this->afi_table ORDER BY icon_extension", ARRAY_A);
			?>
				<script type="text/javascript">
					function showDialogDelete(id) {
						jQuery("#delete_icon_single_value").val(id);
						tb_show("", "#TB_inline?width=200&height=100&inlineId=test&modal=true", true);
					}
				</script>
				<div class="wrap">
					<div class="icon32 icon-afi"><br></div>
					<h2><?php echo esc_html( $title ); ?></h2>
					<div class="clear" style="min-height:15px;"></div>
					<span><b>NOTE:</b> Refer help for instructions to use AF Icons.</span>
					<div class="clear" style="min-height:5px;"></div>
					<div id="dashboard-widgets-wrap">
						<table class="widefat fixed" cellspacing="0" style="width:500px;">
							<thead>
								<tr>
									<th scope="col"><?php _e('File Extension'); ?></th><th scope="col"><?php _e('Icon Available'); ?></th><th scope="col"><?php _e('Action'); ?></th>
								</tr>
							</thead>
							<tfoot>
								<tr>
									<th scope="col" ><?php _e('File Extension'); ?></th><th scope="col" ><?php _e('Icon Available'); ?></th><th scope="col"><?php _e('Action'); ?></th>
								</tr>
							</tfoot>
							<tbody style="width:100%;">
							<?php if($icon_types){
									foreach($icon_types as $icon) {
										$class = ( !isset($class) || $class == 'class="alternate"' ) ? '' : 'class="alternate"';
										?>
										<tr <?php echo $class; ?> >
											<td><?php echo $icon['icon_extension'];	?></td>
											<td><img src="<?php echo $this->plugin_url . '/mime/' . $icon['icon_name'];?>" alt="<?php echo $icon['icon_extension'];	?>" /></td>
											<td><a onclick="showDialogDelete(<?php echo $icon['id'] ?>);" href="#" class="delete"><?php _e('Delete'); ?></a></td>
										</tr>
									<?php }?>	
							<?php }?>
							</tbody>
						</table>
					</div>
				</div>
				
				<div id="delete_icon_single" style="display: none;" >
					<div id="test">
					<form id="form-delete-icon_single" method="POST" accept-charset="utf-8" action="<?php echo admin_url('admin.php?page=afi-overview') ; ?>">
						<input type="hidden" id="delete_icon_single_value" name="TB_iconsingle" value="" />
						<table width="100%" border="0" cellspacing="3" cellpadding="3" >
							<tr valign="top">
								<td><strong><?php _e('Delete Icon?'); ?></strong></td>
							</tr>
						  	<tr align="center">
						    	<td colspan="2" class="submit">
						    		<input class="button-primary" type="submit" name="TB_DeleteSingle" value="<?php _e('OK'); ?>" />
						    		&nbsp;
						    		<input class="button-secondary" type="reset" value="&nbsp;<?php _e('Cancel'); ?>&nbsp;" onclick="tb_remove()"/>
						    	</td>
							</tr>
						</table>
					</form>
					</div>
				</div>
			<?php 
		}
		
		/**
		 * Function to delete data from database in multisite.
		 * @param $id - icon id
		 * @author Praveen Rajan
		 */
		function _afi_delete($id) {
			
			global $wpdb;
			$sub_name_type = 'afi_types';
	        $this->afi_table  = $wpdb->prefix . $sub_name_type;
	        $wpdb->query("DELETE FROM $this->afi_table WHERE id = '$id'");
	        
	        return;
		}
		
		/**
		 * Function show manage page of plugin
		 * @author Praveen Rajan
		 */
		function afi_add() {
			
			$title = __('AF Icons - Add Icons');
			global $wpdb;
			$sub_name_type = 'afi_types';
	        $this->afi_table  = $wpdb->prefix . $sub_name_type;
			
			if(isset($_POST['uploadafi'])){
				$status = true;
				$error_message = '';
				if((trim($_POST['extension_name'])) == ''){
					$error_message .= '<p>File extension left blank.</p>';
					$status = false;
				}	
				$icon_result = $wpdb->get_var("SELECT icon_extension FROM " .  $this->afi_table . " WHERE icon_extension = '". trim($_POST['extension_name']) . "' ");
				if($icon_result){
					$error_message .= '<p>Extension type already exist.</p>';
					$status = false;
				}
				if($_FILES['afifiles']['error'][0] == 4) {
					$error_message .= '<p>No icon file uploaded.</p>';
					$status = false;
				}
				if($status){
					$message = AttachmentFileIcons::upload_icons();
					if($message == 'success'){
						echo '<div class="wrap"><h2></h2><div class="updated fade" id="message"><p>Successfully added icon for file type.</p></div></div>' . "\n";
					}else { 
						echo '<div class="wrap"><h2></h2><div class="error" id="error">' . $message . '</div></div>' . "\n";
					}	
				}else {
					echo '<div class="wrap"><h2></h2><div class="error" id="error">' . $error_message . '</div></div>' . "\n";
				}		
			}
			?>
			<script type="text/javascript">	
				/*
				 * Section to initialise multiple file upload
				 */
				jQuery(document).ready(function(){
					jQuery('#afifiles').MultiFile({
						STRING: {
					    	remove:'[<?php _e('remove') ;?>]',
					    	denied:'File type not permitted.',
			  			},
			  			accept : 'png',
			  			max: 1
				 	});
				});
			</script>
			<div class="wrap">
					<div class="icon32 icon-afi"><br></div>
					<h2><?php echo esc_html( $title ); ?></h2>
					<div class="clear" style="min-height:15px;"></div>
					<span><b>NOTE:</b> Refer help for instructions to use AF Icons.</span>
					<div class="clear" style="min-height:5px;"></div>
					<div id="dashboard-widgets-wrap">
					<form name="uploadafi" id="uploadafi_form" method="POST" enctype="multipart/form-data" action="<?php echo admin_url('admin.php?page=afi-add'); ?>" accept-charset="utf-8" >
						<table class="widefat afi" cellspacing="0" style="width:100%;">
							<thead>
								<tr><th scope="col"></th><th scope="col"></th></tr>
							</thead>
							<tfoot>
								<tr><th scope="col"></th><th scope="col"></th></tr>
							</tfoot>
							<tbody style="width:100%;">
								<tr valign="top"> 
									<th scope="row"><?php _e('Extension of File') ;?></th> 
									<td><input type="text" maxlength="5" size="5" name="extension_name" value="" /></td>
								</tr>
								<tr valign="top"> 
									<th scope="row"><?php _e('Upload Attachment File Icon') ;?></th>
									<td><span id='spanButtonPlaceholder'></span><input type="file" name="afifiles[]" id="afifiles" size="35" class="afifiles"/>
									<br/>
									<i>( <?php _e('Allowed format: png <br/> Allowed dimension: 16 X 16 px') ;?> )</i></td>
								</tr> 
								<tr valign="top">
									<td></td>
									<td>
										<input class="button-primary" type="submit" name="uploadafi" id="uploadafi_btn" value="<?php _e('Upload Icon') ;?>" />
									</td>
								</tr> 
							</tbody>
						</table>
					</form>
					</div>
			</div>
			<?php 		
		}
		
		/**
		 * Function for uploading 
		 * 
		 * @return void
		 * @author Praveen Rajan
		 */
		function upload_icons() {
		
			$afifiles = $_FILES['afifiles'];
			$message = '';
			$status = true;
			if (is_array($afifiles)) {
				foreach ($afifiles['name'] as $key => $value) {
					// look only for uploded files
					if ($afifiles['error'][$key] == 0) {
						$temp_file = $afifiles['tmp_name'][$key];
						$image_info = getimagesize($temp_file);
						if($image_info[0] != 16 || $image_info[1] != 16){
							$message .= '<p>Icon file does not meet the specified dimension.</p>';
							$status = false;
							break;
						}
						//clean filename and extract extension
						$filepart = AttachmentFileIcons::fileinfo( $afifiles['name'][$key] );
						$filename = trim($_POST['extension_name']) . '-icon.' .$filepart['extension'];
						$dest_file = $this->plugin_path . '/mime/' . $filename;
						//check for folder permission
						if ( !is_writeable($this->plugin_path . '/mime/') ) {
							$message .= '<p>Unable to write to directory ' . $this->plugin_path . '/mime/ Is this directory writable by the server?</p>';
							$status = false;
							break;
						}
						// save temp file to gallery
						if ( !@move_uploaded_file($temp_file, $dest_file) ){
							$message .= '<p>Error, the file could not moved to : '.$dest_file.'</p>';
							$status = false;
							break;
						} 
						if ( !AttachmentFileIcons::chmod($dest_file) ) {
							$message .= '<p>Error, the file permissions could not set</p>';
							$status = false;
							break;
						}
					}else {
						$message .= '<p>Error uploading file(Missing \'temp\' folder).</p>';
						$status = false;
						break;
					}
				}
				if($status){
					
					global $wpdb;
			        $sub_name_type = 'afi_types';
			        $this->afi_table  = $wpdb->prefix . $sub_name_type;
			        
					$icon_data = array( 'icon_extension' => trim($_POST['extension_name']), 'icon_name' => $filename );
					
					if (function_exists('is_multisite') && is_multisite()) {
						
						$old_blog = $wpdb->blogid;
						// Get all blog ids
						$blogids = $wpdb->get_col($wpdb->prepare("SELECT blog_id FROM $wpdb->blogs"));
						foreach ($blogids as $blog_id) {
							switch_to_blog($blog_id);
							AttachmentFileIcons::_afi_add($icon_data);
						}
						switch_to_blog($old_blog);
						$status = true;
					}else {
						$wpdb->insert( $this->afi_table, $icon_data ); 
						$status = true;	
					}
					
					if($status)
						return 'success';
				}
			}
			return $message;
		}
		
		/**
		 * Function to add data to database in multisite.
		 * @param $icon_data - icon data
		 * @author Praveen Rajan
		 */
		function _afi_add($icon_data) {
			
			global $wpdb;
	        $sub_name_type = 'afi_types';
	        $this->afi_table  = $wpdb->prefix . $sub_name_type;
	        
	        $wpdb->insert( $this->afi_table, $icon_data ); 
		}
	
		/**
		 * Function to get fileinfo 
		 * 
		 * @param string $name The name being checked. 
		 * @return array containing information about file
		 * author Praveen Rajan
		 */
		function fileinfo( $name ) {
			
			//Sanitizes a filename replacing whitespace with dashes
			$name = sanitize_file_name($name);
			//get the parts of the name
			$filepart = pathinfo ( strtolower($name) );
			if ( empty($filepart) )
				return false;
			// required until PHP 5.2.0
			if ( empty($filepart['filename']) ) 
				$filepart['filename'] = substr($filepart['basename'],0 ,strlen($filepart['basename']) - (strlen($filepart['extension']) + 1) );
			$filepart['filename'] = sanitize_title_with_dashes( $filepart['filename'] );
			$filepart['extension'] = $filepart['extension'];
			//combine the new file name
			$filepart['basename'] = $filepart['filename'] . '.' . $filepart['extension'];
			return $filepart;
		}
		
		/**
		 * Set correct file permissions (taken from wp core)
		 * 
		 * @param string $filename
		 * @return bool $result
		 * @author Praveen Rajan
		 */
		function chmod($filename = '') {
	
			$stat = @ stat(dirname($filename));
			$perms = $stat['mode'] & 0007777;
			$perms = $perms & 0000666;
			if ( @chmod($filename, $perms) )
				return true;
				
			return false;
		}
		
		/**
		 * Function to install afi plugin
		 * @author Praveen Rajan
		 */
		function afi_install(){
			global $wpdb;
			
			if (function_exists('is_multisite') && is_multisite()) {
				// check if it is a network activation - if so, run the activation function for each blog id
				if (isset($_GET['networkwide']) && ($_GET['networkwide'] == 1)) {
					$old_blog = $wpdb->blogid;
					// Get all blog ids
					$blogids = $wpdb->get_col($wpdb->prepare("SELECT blog_id FROM $wpdb->blogs"));
					foreach ($blogids as $blog_id) {
						switch_to_blog($blog_id);
						$this->_afi_activate();
					}
					switch_to_blog($old_blog);
					return;
				}
			}
			$this->_afi_activate();
		}		
		
		/**
		 * Function to create database for plugin.
		 * @author Praveen Rajan
		 */
		function _afi_activate() {
			
			global $wpdb;
	        $sub_name_type = 'afi_types';
	        $this->afi_table  = $wpdb->prefix . $sub_name_type;
			if($wpdb->get_var("SHOW TABLES LIKE '$this->afi_table'") != $this->afi_table) {
				$sql = "CREATE TABLE " . $this->afi_table . " (
						 	  `id` bigint(20) NOT NULL auto_increment,
							  `icon_extension` varchar(255) NOT NULL,
							  `icon_name` mediumtext,
							  PRIMARY KEY  (`id`),
							  UNIQUE KEY `icon_extension` (`icon_extension`)
						);";
				require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
				dbDelta($sql);
			}
			$icons_exist = AttachmentFileIcons::scandir_mime($this->plugin_path . '/mime/');
			$initial_icons = array();
			foreach($icons_exist as $icons) {
				$temp_value = explode('-', $icons);
				$initial_icons[] = array( 'icon_extension' => $temp_value[0], 'icon_name' => $icons );
			}
			foreach($initial_icons as $icon) {
				$wpdb->insert( $this->afi_table, $icon ); 
			}
		}
		
		/**
		 * Function to uninstall plugin
		 * @author Praveen Rajan
		 */
		function afi_uninstall(){
			
			global $wpdb;
			if (function_exists('is_multisite') && is_multisite()) {
			// check if it is a network activation - if so, run the activation function for each blog id
				if (isset($_GET['networkwide']) && ($_GET['networkwide'] == 1)) {
					$old_blog = $wpdb->blogid;
					// Get all blog ids
					$blogids = $wpdb->get_col($wpdb->prepare("SELECT blog_id FROM $wpdb->blogs"));
					foreach ($blogids as $blog_id) {
						switch_to_blog($blog_id);
						$this->_afi_deactivate();
					}
					switch_to_blog($old_blog);
					return;
				}	
			} 
			$this->_afi_deactivate();
		}
		
		/**
		 * Function to delete tables of plugins
		 * @author Praveen Rajan
		 */
		function _afi_deactivate() {
			
			global $wpdb;
			$sub_name_type = 'afi_types';
	        $this->afi_table  = $wpdb->prefix . $sub_name_type;
		  	$wpdb->query("DROP TABLE IF EXISTS $this->afi_table");
		}
		
		
		/**
		 * Function to add document icon to each attachment parsed.
		 * @param $matches - input content
		 * @return string - replaced output string
		 * @author Praveen Rajan
		 */		
		function Attachment_File_Render($matches){
			
			global $wpdb;
			$sub_name_type = 'afi_types';
	        $this->afi_table  = $wpdb->prefix . $sub_name_type;
			$icon_types = $wpdb->get_results("SELECT * FROM  $this->afi_table", ARRAY_A);
			preg_match_all('/([\.\w]*)=(.*?) /i', $matches[1], $attributes);
			$arguments = array();
			$arguments = $this->Attachment_File_Split($matches[1]);
			$file_link = $arguments['href'];
			$file_name_array = explode("/", $file_link);
			$file_name = array_reverse($file_name_array);  //creates an array out of the url and grabs the filename
			$file_title = explode('.', $file_name[0]);
			$file_title = array_reverse($file_title);
			
			$title = trim($arguments['title']);
			foreach($icon_types as $icon) {
				if($icon['icon_extension'] == $file_title[0]){
					$image_name = $icon['icon_name'];
					break;
				}
			}
			$sAttachmentString .= "<div class='documentIcons'>";
			$sAttachmentString .= "<div class='documentIcons_icon'><a href='$file_link'>";
			$sAttachmentString .= "<img src='".$this->plugin_url."/mime/".$image_name."'/>";
			$sAttachmentString .= "</a></div>";
			$sAttachmentString .= "<div class='documentIcons_link'><a href='$file_link'>$title</a></div>";
			$sAttachmentString .= "</div><div class='clear'></div>";
			$image_name = '';
			return $sAttachmentString;
		}
		
		/**
		 * Function to split arguments parsed
		 * @param $argument_string - argument to be split
		 * @return array of argument
		 * @author Praveen Rajan
		 */
		function Attachment_File_Split($argument_string){
			
		    preg_match_all('/(?:[^ =]+?)=(?:["\'].+?["\']|[^ ]+)/', $argument_string, $items);
		    $args = array();
		    foreach ($items[0] as $item){
		        $parts = explode("=", $item);
		        $name = $parts[0];
		        $value = implode("=", array_slice($parts, 1));
		        $args[$name] = strip_tags(trim($value, "\"'"));
		    }
		    return $args;
		}
	
		/**
		 * Function to match the expression
		 * @param $content - input string
		 * @return string - replaced output string
		 * @author Praveen Rajan
		 */
		function Attachment_File_Parse($content) {
			$content = preg_replace_callback("/\[attachment ([^]]*)\/\]/i",  array(&$this, 'Attachment_File_Render'),  $content);
			return $content;
		}
		
		/**
		 * Function to add style to wordpress template.
		 * @return void
		 * @author Praveen Rajan
		 */
		function Attachment_Header_Code(){
			wp_enqueue_script('jquery.multifile', $this->plugin_url . '/jquery.multifile.js', 'jquery');
			wp_enqueue_script('thickbox',null,array('jquery'));
    		echo '<link rel="stylesheet" href="'. site_url() . '/' . WPINC . '/js/thickbox/thickbox.css" type="text/css" media="screen" />';
			echo '<link rel="stylesheet" href="'. $this->plugin_url . '/attachment-file.css" type="text/css" media="screen" />';
		}
		
		/**
		 * Scan folder for icons
		 * 
		 * @param string $dirname
		 * @return array $files list of video filenames
		 * @author Praveen Rajan 
		 */
		function scandir_mime( $dirname ) { 
			$ext = array('png', 'PNG'); 
			$files = array(); 
			if( $handle = opendir( $dirname ) ) { 
				while( false !== ( $file = readdir( $handle ) ) ) {
					$info = pathinfo( $file );
					// just look for video with the correct extension
	                if ( isset($info['extension']) )
					    if ( in_array( strtolower($info['extension']), $ext) )
						   $files[] = utf8_encode( $file );
				}		
				closedir( $handle ); 
			} 
			sort( $files );
			return ( $files ); 
		} 
	}
}

if (class_exists("AttachmentFileIcons")) {
	
	//Creates object of plugin class
	$AttachmentIcons = new AttachmentFileIcons();
}


if (isset($AttachmentIcons)){
	register_activation_hook( __FILE__, array(&$AttachmentIcons,'afi_install') );
	register_deactivation_hook(__FILE__,  array(&$AttachmentIcons,'afi_uninstall'));
}
?>